@@ -43,13 +43,25 @@ except ImportError:
     pass
 
 # =============================================================================
-# Constants
+# Constants and utility functions
 # =============================================================================
 FILES_PATH = os.path.dirname(os.path.realpath(__file__))
 IPYKERNEL_6 = ipykernel.__version__[0] >= '6'
 TIMEOUT = 15
 SETUP_TIMEOUT = 60
 
+# We declare this constant immediately before the test, as determining
+# that TURTLE_ACTIVE is True will briefly pop up a window, similar to the
+# windows that will pop up during the test itself.
+TURTLE_ACTIVE = False
+try:
+    import turtle
+    turtle.Screen()
+    turtle.bye()
+    TURTLE_ACTIVE = True
+except:
+    pass
+
 
 @contextmanager
 def setup_kernel(cmd):
@@ -529,8 +541,11 @@ if __name__ == '__main__':
 
 
 @flaky(max_runs=3)
-@pytest.mark.skipif(not PY3,
-                    reason="Only meant for Python 3")
+@pytest.mark.skipif(
+    not PY3, reason="Only meant for Python 3")
+@pytest.mark.skipif(
+    sys.platform == 'darwin' and sys.version_info[:2] == (3, 8),
+    reason="Fails on Mac with Python 3.8")
 def test_dask_multiprocessing(tmpdir):
     """
     Test that dask multiprocessing works on Python 3.
@@ -650,6 +665,9 @@ def test_runfile(tmpdir):
 
 
 @flaky(max_runs=3)
+@pytest.mark.skipif(
+    sys.platform == 'darwin' and sys.version_info[:2] == (3, 8),
+    reason="Fails on Mac with Python 3.8")
 def test_np_threshold(kernel):
     """Test that setting Numpy threshold doesn't make the Variable Explorer slow."""
 
@@ -716,21 +734,10 @@ f = np.get_printoptions()['formatter']
         assert "{'float_kind': <built-in method format of str object" in content
 
 
-# We declare this constant immediately before the test, as determining
-# that TURTLE_ACTIVE is True will briefly pop up a window, similar to the
-# windows that will pop up during the test itself.
-TURTLE_ACTIVE = False
-try:
-    import turtle
-    turtle.Screen()
-    turtle.bye()
-    TURTLE_ACTIVE = True
-except:
-    pass
-
 @flaky(max_runs=3)
-@pytest.mark.skipif(not TURTLE_ACTIVE,
-                    reason="Doesn't work on non-interactive settings or Python installations without Tk")
+@pytest.mark.skipif(
+    not TURTLE_ACTIVE,
+    reason="Doesn't work on non-interactive settings or Python without Tk")
 def test_turtle_launch(tmpdir):
     """Test turtle scripts running in the same kernel."""
     # Command to start the kernel
@@ -1181,5 +1188,37 @@ def test_debug_namespace(tmpdir):
                     break
 
 
+def test_non_strings_in_locals(kernel):
+    """
+    Test that we can hande non-string entries in `locals` when bulding the
+    namespace view.
+
+    This is a regression test for issue spyder-ide/spyder#19145
+    """
+    if IPYKERNEL_6:
+        execute = asyncio.run(kernel.do_execute(
+            'locals().update({1:2})', True))
+    else:
+        execute = kernel.do_execute('locals().update({1:2})', True)
+
+    nsview = repr(kernel.get_namespace_view())
+    assert "1:" in nsview
+
+
+@pytest.mark.skipif(
+    sys.version_info[0] < 3, reason="Doesn't work with Python 2")
+def test_django_settings(kernel):
+    """
+    Test that we don't generate errors when importing `django.conf.settings`.
+
+    This is a regression test for issue spyder-ide/spyder#19516
+    """
+    execute = asyncio.run(kernel.do_execute(
+        'from django.conf import settings', True))
+
+    nsview = repr(kernel.get_namespace_view())
+    assert "'settings':" in nsview
+
+
 if __name__ == "__main__":
     pytest.main()
